@@ -371,8 +371,12 @@ Read Logbook: %s\n\
   %d records\n\
 ================================================\n",
 fname, db->nbrRecs());
-	REQ(write_rxtext, szmsg);
+
+	if (progdefaults.DisplayLogbookRead && (db == &qsodb))
+		REQ(write_rxtext, szmsg);
+
 	LOG_INFO("%s", szmsg);
+
 	if (num_read_errors) {
 		if (!read_errors.empty()) {
 			read_errors.append("\n");
@@ -386,7 +390,7 @@ fname, db->nbrRecs());
 ================================================\n").append(szmsg);
 			read_errors.append("\
 ================================================\n");
-		if (db == &qsodb) REQ(write_rxtext, read_errors.c_str());
+		REQ(write_rxtext, read_errors.c_str());
 	}
 
 	if (db == &qsodb)
